/*******************************************************************
*
*  DESCRIPTION: Atomic Model Display
*
*
*******************************************************************/

/** include files **/
#include "display.h"    // class Display
#include <stdlib.h>   
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )

/** public functions **/

/*******************************************************************
* Function Name: Display
* Description: 
********************************************************************/
Display::Display( const string &name )
: Atomic( name )
, in( addInputPort( "in" ) )
, displayTime( 0, 0, 5, 0 )
{
	string time( MainSimulator::Instance().getParameter( description(), "displayTime" ) );

	if( time != "" )
		displayTime = time ;

 	string cantMsg( MainSimulator::Instance().getParameter( description(), "cantMsgs" ) );

	if ( cantMsg == "" ) {
		cantMsgs = 0;
		msgs = NULL;			
	}	
	else {
		cantMsgs = atoi( cantMsg.data() );
		msgs = new string [cantMsgs];	
		// Levanto los mensajes a mostrar del archivo de configuracion
		_initArrayMsgs();	
	}

	openLogFile();
}

/*******************************************************************
* Function Name: ~Display
* Description: 
********************************************************************/
Display::~Display()
{
	if( msgs ) 
		delete msgs;
	
	if( logFile );
		fclose( logFile );

}


/*******************************************************************
* Function Name: initFunction
* Precondition: El tiempo del proximo evento interno es Infinito
********************************************************************/
Model &Display::initFunction()
{
	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: 
********************************************************************/
Model &Display::externalFunction( const ExternalMessage &msg )
{
        // muestra siempre el ltimo mensaje que le llega	
	if( msg.port() == in )
	{
	        idMensaje = (int)msg.value();	
	        if ( 0 > idMensaje || idMensaje >= cantMsgs ) {
			MException e( string( "Mensaje no reconocido: " ) + msg.value() );	
		 	e.addLocation( MEXCEPTION_LOCATION() );	
			throw e;	
	 	}	
		loguearMensaje( msg );	
		holdIn( active, displayTime );
	}
	return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: 
********************************************************************/
Model &Display::internalFunction( const InternalMessage & )
{
	
	passivate();
	return *this ;
}

/*******************************************************************
* Function Name: outputFunction
* Description: 
********************************************************************/
Model &Display::outputFunction( const InternalMessage &msg )
{
	return *this ;
}

/*******************************************************************
* Function Name: _initArrayMsgs
* Description: 
********************************************************************/
Model &Display::_initArrayMsgs()
{
  
	if ( msgs ) {	
		for( int i = 0; i < cantMsgs; i++ ) {
			msgs[i] = string( MainSimulator::Instance().getParameter( description(), string( "msg" ) + i ) );
		}
	}
	return *this;

}

/*******************************************************************
* Function Name: openLogFile
* Description: 
********************************************************************/
void Display::openLogFile()
{
	string logFileName( MainSimulator::Instance().getParameter( description(), "logFileName" ) );

	if( logFileName != "" )
		logFileName = "display.log" ;
	
	logFile = fopen( logFileName.data(), "w+" );	

	if( !logFile ) {
		MException e( string( "Error al abrir el archivo de log: " ) + logFileName );	
	 	e.addLocation( MEXCEPTION_LOCATION() );	
		throw e;	
	
	}
}

/*******************************************************************
* Function Name: loguearMensaje
* Description: 
********************************************************************/
Model &Display::loguearMensaje( const ExternalMessage &msg )
{
	string mensaje; 
	if ( msgs ) {
		mensaje = msgs[idMensaje];	
	}	
	else {
		mensaje = "Error en display de mensaje";	
	}
	fprintf( logFile, "\n----------------------------------" );	
	fprintf( logFile, "\nMensaje: %s", mensaje.data() );	
  	fprintf( logFile, "\nHora impresion: %s", msg.time().asString().data() );
	return *this;
} 
