/*******************************************************************
*
*  DESCRIPTION: Atomic Model Generador
*
*
*******************************************************************/

/** include files **/
#include "generador.h"    // class Generador
#include <stdlib.h>   
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )


/** public functions **/

/*******************************************************************
* Function Name: Generador
* Description: 
********************************************************************/
Generador::Generador( const string &name )
: Atomic( name )
, nuevo_boleto( addInputPort( "nuevo_boleto" ) )
, moneda_verificada( addInputPort( "moneda_verificada" ) )
, cancelar( addInputPort( "cancelar" ) )
, activar_vm( addOutputPort( "activar_vm" ) )
, dar_vuelto( addOutputPort( "dar_vuelto" ) )
, impresion( addOutputPort( "impresion" ) )
, display( addOutputPort( "display" ) )
, tout( 1, 0, 0, 0 )
{


}

/*******************************************************************
* Function Name: initFunction
* Precondition: El tiempo del proximo evento interno es Infinito
********************************************************************/
Model &Generador::initFunction()
{
	fase = pasivo;
	passivate();
	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: 
********************************************************************/
Model &Generador::externalFunction( const ExternalMessage &msg )
{
	if( msg.port() == nuevo_boleto ) {
		if( fase == pasivo ) {
			//chequea si debe activar la verificacion
			nuevoBoleto= (float) msg.value();
			importeActual = (float) 0;
			fase = activando;
			holdIn( active, Time::Zero );	
		} 
	}
	
	if( msg.port() == cancelar )
	{
		if( state() == active ) {
			fase = devolver_dinero;
			holdIn( active, Time::Zero );
		}
	}

	if( msg.port() == moneda_verificada )
	{
		if( state() == active && fase == waiting ) {
			importeActual += (float) msg.value();
//			printf ( "actual %f /boleto %f \n", importeActual, nuevoBoleto);
			if (importeActual >= nuevoBoleto) { 
				if (importeActual == nuevoBoleto)  {
					fase = imprimiendo;
					printf( "imprimiendo\n");
				}
				else {
					fase = dando_vuelto;
				}
			}
			else 
				fase = actualizando_display;

			holdIn ( active, Time::Zero );
		}
				

	}
	return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: 
********************************************************************/
Model &Generador::internalFunction( const InternalMessage & )
{
	if( fase == waiting || fase == imprimiendo || fase == devolver_dinero ) {
		// di timeout o termin
		fase = pasivo;
		passivate();

	} else if( fase == activando || fase == actualizando_display ) {
		fase = waiting;
		holdIn( active, tout);	

	} else if( fase == dando_vuelto ) {
		fase = imprimiendo;
		holdIn( active, Time(0,0,0,1));	
	}
	
	return *this ;
}

/*******************************************************************
* Function Name: outputFunction
* Description: 
********************************************************************/
Model &Generador::outputFunction( const InternalMessage &msg )
{
	if( fase == waiting ) {
		sendOutput( msg.time(), display, 0 );	
		sendOutput( msg.time(), activar_vm, 0 );

	} else if( fase == actualizando_display ) {
		sendOutput( msg.time(), display, -importeActual);

	} else if( fase == imprimiendo ) {
		sendOutput( msg.time(), display, 1);
		sendOutput( msg.time(), impresion, -nuevoBoleto);
		sendOutput( msg.time(), activar_vm, 0 );

	} else if( fase == dando_vuelto ) {
		sendOutput( msg.time(), dar_vuelto, importeActual - nuevoBoleto);

	} else if( fase == devolver_dinero ) {
		sendOutput( msg.time(), dar_vuelto, importeActual );
		sendOutput( msg.time(), activar_vm, 0 );

	} else if( fase == activando ) {
		sendOutput( msg.time(), activar_vm, 1 );

	}


	return *this ;
}
