/*******************************************************************
*
*  DESCRIPTION: Atomic Model Printer
*
*
*******************************************************************/

/** include files **/
#include "printer.h"    // class Printer
#include <stdlib.h>   
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )

#define	MAX_MSGS	2

char *msgs[] = { "mgs1", "msg2" };

/** public functions **/

/*******************************************************************
* Function Name: Printer
* Description: 
********************************************************************/
Printer::Printer( const string &name )
: Atomic( name )
, in( addInputPort( "in" ) )
, stop( addOutputPort( "stop" ) )
, done( addOutputPort( "done" ) )
, printTime( 0, 0, 5, 0 )
{
	string time( MainSimulator::Instance().getParameter( description(), "printTime" ) );


	if( time != "" )
		printTime = time ;


	openLogFile();

}

/*******************************************************************
* Function Name: initFunction
* Precondition: El tiempo del proximo evento interno es Infinito
********************************************************************/
Model &Printer::initFunction()
{
	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: 
********************************************************************/
Model &Printer::externalFunction( const ExternalMessage &msg )
{
	if( msg.port() == in )
	{
		if( state() == passive ) {	
	        	idMensaje = (int)msg.value();	
	        	if ( 0 > idMensaje || idMensaje >= MAX_MSGS ) {
				MException e( string( "Mensaje no reconocido: " ) + msg.value() );	
		 		e.addLocation( MEXCEPTION_LOCATION() );	
				throw e;	
	 		}
			// stopea la cola de impresion	
			fase = stoping;	
			loguearMensaje( msg );		
			holdIn( active, Time::Zero );
		}	
	}
	return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: 
********************************************************************/
Model &Printer::internalFunction( const InternalMessage & )
{
	if( fase == Printer::stoping ) {
		// reanuda la cola	
		fase = Printer::printing;	
		holdIn( active, printTime );	
	} else {
		if( fase == Printer::printing ) {
			passivate();
		}
	}
	return *this ;
}

/*******************************************************************
* Function Name: outputFunction
* Description: 
********************************************************************/
Model &Printer::outputFunction( const InternalMessage &msg )
{
	if( fase == stoping ) {
		sendOutput( msg.time(), stop, 0 );	
	}	
	if( fase == printing ) {
		sendOutput( msg.time(), done, 0 );	
	}	
	return *this ;
}

/*******************************************************************
* Function Name: openLogFile 
* Description: 
********************************************************************/
void Printer::openLogFile()
{
  
	string logFileName( MainSimulator::Instance().getParameter( description(), "logFileName" ) );

	if( logFileName != "" )
		logFileName = "printer.log" ;
	
	logFile = fopen( logFileName.data(), "w+" );	

	if( !logFile ) {
		MException e( string( "Error al abrir el archivo de log: " ) + logFileName );	
	 	e.addLocation( MEXCEPTION_LOCATION() );	
		throw e;	
	
	}
}

/*******************************************************************
* Function Name: loguearMensaje
* Description: 
********************************************************************/
Model &Printer::loguearMensaje( const ExternalMessage &msg )
{
	fprintf( logFile, "\n----------------------------------" );
 	fprintf( logFile, "\nMensaje: %s", msgs[idMensaje] );
 	fprintf( logFile, "\nHora impresion: %s", msg.time().asString().data() );
	return *this;
} 
