/*******************************************************************
*
*  DESCRIPTION: Atomic Model Verificador
*
*
*******************************************************************/

/** include files **/
#include "verificador.h"    // class Verificador
#include <stdlib.h>   
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )


/** public functions **/

/*******************************************************************
* Function Name: Verificador
* Description: 
********************************************************************/
Verificador::Verificador( const string &name )
: Atomic( name )
, in( addInputPort( "in" ) )
, activar_vm( addInputPort( "activar_vm" ) )
, stop( addOutputPort( "stop" ) )
, done( addOutputPort( "done" ) )
, moneda_verificada( addOutputPort( "moneda_verificada" ) )
, tout_verificacion( 1, 0, 0, 0 )
{
	string tout( MainSimulator::Instance().getParameter( description(), "tout" ) );


	if( tout != "" )
		tout_verificacion = tout ;


}

/*******************************************************************
* Function Name: initFunction
* Precondition: El tiempo del proximo evento interno es Infinito
********************************************************************/
Model &Verificador::initFunction()
{
	fase = starting;
	passivate();
	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: 
********************************************************************/
Model &Verificador::externalFunction( const ExternalMessage &msg )
{
	if( msg.port() == activar_vm ) {
		if( state() == passive ) {
			//chequea si debe activar la verificacion
			if ( msg.value() ) {
			 	fase = starting;
				holdIn( active, Time::Zero );	
			}		
		} 
		else {
			// si le llega un mensaje de
			// desactivar, desactiva...	
			if( !msg.value() ) {
				fase = stopping;	
				holdIn( active, Time::Zero );	
			}
		}


	}
	
	if( msg.port() == in )
	{
		if( state() == active ) {
			if( fase == waiting ) {	
				valorMoneda = (float) msg.value();	
				// stopea la cola 	
				fase = verificar;	
				holdIn( active, Time::Zero );
			}	
		}
	}
	return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: 
********************************************************************/
Model &Verificador::internalFunction( const InternalMessage & )
{
	if( fase == stopping || fase == waiting ) {
		passivate();

	} else if( fase == verificar ) {
		fase = verificando;
		Time t = string("0:0:1:0");
		holdIn( active, t ) ;	

	} else if( fase == verificando || fase == starting ) {
		// vuelvo a esperar otra moneda a verificar 
		fase = waiting;	
		holdIn( active, tout_verificacion );	

	}
	
	return *this ;
}

/*******************************************************************
* Function Name: outputFunction
* Description: 
********************************************************************/
Model &Verificador::outputFunction( const InternalMessage &msg )
{
	if( fase == starting ) {
//		sendOutput( msg.time(), done, 0 );	
	}	
 	if( fase == stopping || fase == waiting ) {
		sendOutput( msg.time(), done, 0 );	
	}	

	if( fase == verificar ) {
		sendOutput( msg.time(), stop, 1 );	
	}	

	if( fase == verificando ) {
		if( valorMoneda == 0.25 || valorMoneda == (float) 0.10 ||
		    valorMoneda == (float) 0.05 || valorMoneda == 0.50 ) {
			sendOutput( msg.time(), moneda_verificada, valorMoneda );
		}
		sendOutput( msg.time(), done, 0 );	
	}	

	return *this ;
}
