/*******************************************************************
*
*  DESCRIPTION: Atomic Model Vuelto
*
*
*******************************************************************/

/** include files **/
#include "vuelto.h"      // class Vuelto
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )
#include <stdlib.h>   

/** public functions **/

/*******************************************************************
* Function Name: Vuelto
* Description: 
********************************************************************/
Vuelto::Vuelto( const string &name )
: Atomic( name )
, in( addInputPort( "in" ) )
, vuelto( addOutputPort( "vuelto" ) )
, resto( addOutputPort( "resto" ) )
, preparationTime( 0, 0, 10, 0 )
{

 	string moneda( MainSimulator::Instance().getParameter( description(), "valorMoneda") );
	
	if ( moneda == "" ) 
	  valorMoneda = 0.10; 
	
	else 
	  valorMoneda = atof ( moneda.data() ) ;

 	string stock( MainSimulator::Instance().getParameter( description(), "stockMonedas") );
	
	if ( stock == "" ) 
	  stockMonedas = 10; 
	
	else 
	  stockMonedas = atoi( stock.data() );
  
		
	string time( MainSimulator::Instance().getParameter( description(), "preparation" ) ) ;

	if( time != "" )
		preparationTime = time ;

	
}

/*******************************************************************
* Function Name: initFunction
* Precondition: El tiempo del proximo evento interno es Infinito
********************************************************************/
Model &Vuelto::initFunction()
{
	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: 
********************************************************************/
Model &Vuelto::externalFunction( const ExternalMessage &msg )
{
	if( msg.port() == in && state() == passive )
	{
			acumVuelto = msg.value();
			holdIn( active, Time::Zero );
	}

	return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: 
********************************************************************/
Model &Vuelto::internalFunction( const InternalMessage & )
{
	
	if ( stockMonedas > 0 && acumVuelto >= valorMoneda ) {
		stockMonedas--;
		acumVuelto -= valorMoneda;
		holdIn( active, preparationTime);
	} else
		passivate();
	

	return *this ;
}

/*******************************************************************
* Function Name: outputFunction
* Description: 
********************************************************************/
Model &Vuelto::outputFunction( const InternalMessage &msg )
{
//	printf ( "acumVuelto - valorMoneda: %f - %f\n", acumVuelto, valorMoneda); 
	if ( stockMonedas > 0 && acumVuelto >= valorMoneda ) 
		sendOutput( msg.time(), vuelto, valorMoneda ) ;
	else
	   	sendOutput( msg.time(), resto, acumVuelto );

	return *this ;
}
